#include "RefinementManager.h"
#include "Mesh.h"
#include "Traverse.h"
#include "ElInfo.h"
#include "DOFAdmin.h"
#include "AdaptStationary.h"
#include "AdaptInstationary.h"
#include "FixVec.h"
#include "RCNeighbourList.h"
#include "ProblemStatBase.h"
#include "PeriodicBC.h"

namespace AMDiS {

  RefinementManager* RefinementManager::traversePtr = NULL;
  bool RefinementManager::doMoreRecursiveRefine = false;
  int RefinementManager::callRefineInterpol = 0;

  RCNeighbourList* RefinementManager3d::refList = NULL;

  int RefinementManager::globalMark = 0;

  int RefinementManager::globalRefineFct(ElInfo* elinfo)
  {
    FUNCNAME("RefineManager::globalRefineFct");

    elinfo->getElement()->setMark(globalMark);
    return 0;
  }

  Flag RefinementManager::globalRefine(Mesh *aMesh, int mark)
  {
    FUNCNAME("RefinementManager::globalRefine()");

    if(mark <= 0) return static_cast<Flag>(0);
    globalMark = mark;
    aMesh->traverse(-1, 
		    Mesh::CALL_LEAF_EL |
		    Mesh::FILL_COORDS |
		    Mesh::FILL_BOUND, 
		    globalRefineFct);
    return refineMesh(aMesh);
  }




  Flag RefinementManager::refineMesh(Mesh *aMesh)
  {
    FUNCNAME("RefinementManager::refineMesh()");

    mesh = aMesh;

    //   if(mesh->getDim() == 3) {
    //     ::std::map<BoundaryType, PeriodicBC*> &periodicBCs = 
    //       mesh->getPeriodicBCMap();

    //     ::std::map<BoundaryType, PeriodicBC*>::iterator it;
    //     ::std::map<BoundaryType, PeriodicBC*>::iterator end = periodicBCs.end();
    //     for(it = periodicBCs.begin(); it != end; ++it) {
    //       it->second->updateAssociatedDOFs();
    //     }
    //   }

    int n_elements = mesh->getNumberOfLeaves();
    ElInfo *el_info;

    newCoords  = false;

    stack = NEW TraverseStack;
    doMoreRecursiveRefine = true;
    while (doMoreRecursiveRefine)
      {
	doMoreRecursiveRefine = false;
	el_info = stack->traverseFirst(mesh, -1, 
				       Mesh::CALL_LEAF_EL | Mesh::FILL_NEIGH| Mesh::FILL_BOUND);
	while (el_info)
	  {
	    if (el_info->getElement()->getMark() > 0)
	      {
		doMoreRecursiveRefine = doMoreRecursiveRefine || 
		  (el_info->getElement()->getMark() > 1);
		el_info = refineFunction(el_info);
	      }
	    el_info = stack->traverseNext(el_info);
	  }
      }
  
    if(newCoords) {
      setNewCoords(); // call of sub-class method  
    }

    DELETE stack;

    n_elements = mesh->getNumberOfLeaves() - n_elements;

    //   if(mesh->getDim() == 3) {
    //     ::std::map<BoundaryType, PeriodicBC*> &periodicBCs = 
    //       mesh->getPeriodicBCMap();

    //     ::std::map<BoundaryType, PeriodicBC*>::iterator it;
    //     ::std::map<BoundaryType, PeriodicBC*>::iterator end = periodicBCs.end();
    //     for(it = periodicBCs.begin(); it != end; ++it) {
    //       it->second->clearPeriodicElementData();
    //     }
    //   }

    return(n_elements ? MESH_REFINED : Flag(0));
  }


  // void RefinementManager::refineElement(Element *element)
  // {
  //   FUNCNAME("RefinementManager::refineElement");
  //   element->incrementMark();
  //   refineMesh();
  // }

}
