#include <stdio.h>
#include <string>
#include <fstream>

#include "VtkWriter.h"
#include "DataCollector.h"
#include "DOFVector.h"
#include "SurfaceRegion_ED.h"
#include "ElementRegion_ED.h"

namespace AMDiS { 
  int VtkWriter::writeFile(::std::vector<DataCollector*> *dc,
			   const char *name)
  {
    ::std::ofstream file;
    ::std::list<ElementInfo> *elements = (*dc)[0]->getElementInfos();

    DOFVector< ::std::list<VertexInfo> > *vertexInfos = (*dc)[0]->getVertexInfos();

    int nv = (*dc)[0]->getNumberVertices();
    int ne = (*dc)[0]->getNumberElements();
    int vertices = (*dc)[0]->getMesh()->getGeo(VERTEX);

    file.open(name);

    file << "<?xml version=\"1.0\"?>" << ::std::endl;
    file << "<VTKFile type=\"UnstructuredGrid\" version=\"0.1\" byte_order=\"LittleEndian\">" << ::std::endl;
    file << "  <UnstructuredGrid>" << ::std::endl;
    file << "    <Piece NumberOfPoints=\"" << nv << "\" NumberOfCells=\"" << ne << "\">" << ::std::endl;
    file << "      <Points>" << ::std::endl;
    file << "        <DataArray type=\"Float32\" NumberOfComponents=\"3\" format=\"ascii\">" << ::std::endl;


    DOFVector< ::std::list<VertexInfo> >::Iterator it(vertexInfos, USED_DOFS);
    int counter = 0;

    // for all DOFs
    for (it.reset(); !it.end(); ++it) {
      // for all vertex infos of this DOF
      ::std::list<VertexInfo>::iterator it2;
      for (it2 = it->begin(); it2 != it->end(); ++it2) {
	it2->outputIndex = counter++;
	for (int i = 0; i < Global::getGeo(WORLD); i++) {
	  file << " " << ::std::scientific << it2->coords[i];
	}
	for (int i = Global::getGeo(WORLD); i < 3; i++) {
	  file << " 0.0";
	}
	file << ::std::endl;
      }
    }

    file << "        </DataArray>" << ::std::endl;
    file << "      </Points>" << ::std::endl;
    file << "      <Cells>" << ::std::endl;

    file << "        <DataArray type=\"Int32\" Name=\"offsets\">" << ::std::endl;
    for (int i = 0; i < ne; i++) {
      file << " " << (i + 1) * vertices << ::std::endl;
    }
    file << "        </DataArray>" << ::std::endl;

    file << "        <DataArray type=\"UInt8\" Name=\"types\">" << ::std::endl;
    for (int i = 0; i < ne; i++) {
      switch (vertices) {
      case 2:
	file << " 3" << ::std::endl;
	break;
      case 3: 
	file << " 5" << ::std::endl;
	break;
      case 4:
	file << " 10" << ::std::endl;
	break;		    
      default:
	break;
      }	   
    }
    file << "        </DataArray>" << ::std::endl;

    file << "        <DataArray type=\"Int32\" Name=\"connectivity\">" << ::std::endl;
    ::std::list<ElementInfo>::iterator elementIt;
    
    for (elementIt = elements->begin(); elementIt != elements->end(); ++elementIt) {
      // for all vertices
      for (int i = 0; i < vertices; i++) {
	file << " " << elementIt->vertexInfo[i]->outputIndex;
      }
      file << ::std::endl;
    }
    file << "        </DataArray>" << ::std::endl;
    
    file << "      </Cells>" << ::std::endl;
    file << "      <PointData>" << ::std::endl;

    /* ----- write vertex values -----*/
    for (int i = 0; i < static_cast<int>(dc->size()); i++) {
      file << "        <DataArray type=\"Float32\" Name=\"value" << i 
	   << "\" format=\"ascii\">" << ::std::endl;

      DOFVector<int> *interpPointInd = (*dc)[i]->getInterpPointInd();
      DOFVector<double> *values = (*dc)[i]->getValues();
      DOFVector< ::std::list<WorldVector<double> > > *dofCoords = (*dc)[i]->getDofCoords();

      DOFVector<int>::Iterator intPointIt(interpPointInd, USED_DOFS);
      DOFVector<double>::Iterator valueIt(values, USED_DOFS);
      DOFVector< ::std::list<WorldVector<double> > >::Iterator coordIt(dofCoords, USED_DOFS);
      
      for (intPointIt.reset(), valueIt.reset(), coordIt.reset();
	   !intPointIt.end(); 
	   ++intPointIt, ++valueIt, ++coordIt) 
	{
	  if (*intPointIt == -2) {
	    for (int j = 0; j < static_cast<int>(coordIt->size()); j++) {
	      if (*valueIt < 1e-40) {
		file << " " << 0.0 << ::std::endl;
	      } else {
		file << " " << *valueIt << ::std::endl;
	      }
	    }
	  }
	}
      
      file << "        </DataArray>" << ::std::endl;
    }

    file << "      </PointData>" << ::std::endl;
    file << "    </Piece>" << ::std::endl;
    file << "  </UnstructuredGrid>" << ::std::endl;
    file << "</VTKFile>" << ::std::endl;
    
    file.close();

    return 0;
  }

  int VtkWriter::updateAnimationFile(::std::string valueFilename,
				     ::std::vector< ::std::string > *paraViewAnimationFrames,
				     const char *animationFilename)
  {
    size_t found = valueFilename.find_last_of("/\\");
    paraViewAnimationFrames->push_back(valueFilename.substr(found + 1));

    ::std::ofstream file;
    file.open(animationFilename);

    file << "<?xml version=\"1.0\"?>" << ::std::endl;
    file << "<VTKFile type=\"Collection\" version=\"0.1\" >"  << ::std::endl;
    file << "<Collection>" << ::std::endl;

    int counter = 0;
    ::std::vector< ::std::string >::iterator it;
    for (it = paraViewAnimationFrames->begin(); 
	 it < paraViewAnimationFrames->end(); 
	 ++it, counter++) {
      file << "<DataSet timestep=\"" << counter
	   << "\" part=\"0\" file=\"" << (*it) << "\"/>" << ::std::endl;      
    }

    file << "</Collection>" << ::std::endl;
    file << "</VTKFile>" << ::std::endl;

    file.close();
    
    return 0;
  }
}
